import numpy as np
import pandas as pd
import datetime
from pathlib import Path
import sys
import matplotlib.pyplot as plt
from machineLearningFunctions import *

numRunsSM = int(sys.argv[1])
numRunsDM = int(sys.argv[2])

# SM Data Files
pathSM = '/home/mtde226/MESA_emulator/LOGS/LOGS_'+str(numRunsSM)+'/postProcessResults/'
outputFilePathSM = '/home/mtde226/MESA_emulator/LOGS/LOGS_'+str(numRunsSM)+'/MachineLearningResults/'
inputFileSM = open(pathSM+'iBandOutput.txt', 'r')
correctionFileSM = open(pathSM+'colourCorrections.txt', 'r')
timingFileSM = open(pathSM+'runtimesHost.txt', 'r')

# DM Data Files
pathDM = '/home/mtde226/MESA_emulator/LOGS/LOGS_'+str(numRunsDM)+'/postProcessResults/'
outputFilePathDM = '/home/mtde226/MESA_emulator/LOGS/LOGS_'+str(numRunsDM)+'/MachineLearningResults/'
inputFileDM = open(pathDM+'iBandOutput.txt', 'r')
correctionFileDM = open(pathDM+'colourCorrections.txt', 'r')
timingFileDM = open(pathDM+'runtimesHost.txt', 'r')
outputPathDM = pathDM+'fulldata.txt'
dataColsDM = ['flag','iflag','i','j','k','l','mass','Y','Z','A','time','grav','Teff','FeH','logL','IBand','Ierr','VI','VIerr']
normalColsDM = ['mass','Y','Z','A','time','grav','Teff','FeH','logL','IBand','Ierr','VI','VIerr']

splicediBandOutputpath = pathDM+'iBandOutputSpliced.txt'
outputComboPath = pathDM+'fullComboData.txt'
normaliseComboFile = open(outputFilePathDM+'normalisationConstantsCombo.txt', 'w')

dataSM = np.zeros((numRunsSM, len(dataColsDM)))
dataDM = np.zeros((numRunsDM, len(dataColsDM)))

smMatrix = np.zeros((45, 45, 59))
dmMatrix = np.zeros((15, 10, 25, 30))

# Read SM Data
# Read timing data
lineNum = 0
for line in timingFileSM.readlines():
    temp = line.strip().split()
    dataSM[lineNum, 10] = temp[len(temp)-1]
    lineNum += 1

# Read MESA data
lineNum = 0
for line in inputFileSM.readlines():
    temp = line.strip().split()
    dataSM[lineNum, 0] = temp[0]
    dataSM[lineNum, 2:5] = temp[1:4]
    dataSM[lineNum, 5] = int(-1)
    dataSM[lineNum, 6:9] = temp[4:7]
    dataSM[lineNum, 9] = 0
    dataSM[lineNum, 11:15] = temp[7:11]
    lineNum += 1

# Read bolometric correction data
lineNum = 0
for line in correctionFileSM.readlines():
    temp = line.strip().split(', ')
    dataSM[lineNum, 1] = temp[1]
    dataSM[lineNum, -4:] = temp[5:]
    lineNum += 1
    
# Read DM Data
# Read timing data
lineNum = 0
for line in timingFileDM.readlines():
    temp = line.strip().split()
    dataDM[lineNum, 10] = temp[len(temp)-1]
    lineNum += 1

# Read MESA data
lineNum = 0
for line in inputFileDM.readlines():
    temp = line.strip().split()
    dataDM[lineNum, 0] = temp[0]
    dataDM[lineNum, 2:10] = temp[1:9]
    dataDM[lineNum, 11:15] = temp[9:13]
    lineNum += 1

# Read bolometric correction data
lineNum = 0
for line in correctionFileDM.readlines():
    temp = line.strip().split(', ')
    dataDM[lineNum, 1] = temp[1]
    dataDM[lineNum, -4:] = temp[6:]
    lineNum += 1
    
dataSM = pd.DataFrame(data=dataSM, columns=dataColsDM)
dataDM = pd.DataFrame(data=dataDM, columns=dataColsDM)

massesSM = np.linspace(0.7, 2.25, 46)
helium_abundancesSM = np.linspace(0.2, 0.3, 46)
metallicitiesSM = np.logspace(-1.39794000867, -5, 59)

massesDM = np.unique(dataDM['mass'])
helium_abundancesDM = np.unique(dataDM['Y'])
metallicitiesDM = np.unique(dataDM['Z'])
axions = np.unique(dataDM['A'])

mass_SM_indices = np.zeros(len(massesDM))
helium_SM_indices = np.zeros(len(helium_abundancesDM))
metallicities_SM_indices = np.zeros(len(metallicitiesDM))

mass_SM_DM_dict = {}
helium_SM_DM_dict = {}
metallicities_SM_DM_dict = {}

for i in range(len(massesDM)):
    mass_SM_indices[i] = np.argmin(abs(massesSM - massesDM[i]))
    mass_SM_DM_dict[mass_SM_indices[i]] = i
    
for i in range(len(helium_abundancesDM)):
    helium_SM_indices[i] = np.argmin(abs(helium_abundancesSM - helium_abundancesDM[i]))
    helium_SM_DM_dict[helium_SM_indices[i]] = i

for i in range(len(metallicitiesDM)):
    metallicities_SM_indices[i] = np.argmin(abs(metallicitiesSM - metallicitiesDM[i]))
    metallicities_SM_DM_dict[metallicities_SM_indices[i]] = i
            
datapoints = [[int(i), int(j), int(k)] for i in mass_SM_indices for j in helium_SM_indices for k in metallicities_SM_indices]

for point in datapoints[::-1]:
    i = point[0]
    j = point[1]
    k = point[2]
    transfer_data = dataSM.query('i == @i and j == @j and k == @k').copy(deep=True)
    transfer_data['i'] = int(mass_SM_DM_dict[i])
    transfer_data['j'] = int(helium_SM_DM_dict[j])
    transfer_data['k'] = int(metallicities_SM_DM_dict[k])
    dataDM = pd.concat([transfer_data, dataDM], axis=0, ignore_index=True, sort=False)
    
# dataDM = pd.concat([dataDM, dataSM])    
# dataDM.query('flag != 1 and Teff < 10000', inplace=True)

# plt.figure()
# plt.hist(dataDM['Teff'])
# plt.yscale('log')
# plt.show()
'''
plt.figure()
plt.hist2d(dataDM.dropna()['VI'], dataDM.dropna()['IBand'], bins=20)
plt.xlabel('V-I')
plt.ylabel(r'$M_I$ Band')
plt.title(r'$M_I$ as a function of Colour')
plt.colorbar()
plt.gca().invert_yaxis()
plt.savefig(pathDM+'colourHistCombo.png')
'''

dataDM.to_csv(splicediBandOutputpath, header=False, index=False)

dataDM.query('flag != 1', inplace=True)

print(np.cov(dataDM['IBand'], dataDM['VIerr']))

for column in normalColsDM:
    dataDM[column], _, _, _ = normalise(dataDM[column], normaliseComboFile)
    
dataDM.to_csv(outputComboPath, header=True, index=False)
